﻿using Buddy.Coroutines;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Managers;
using System.Linq;
using System.Threading.Tasks;
using UltimaCR.Spells.Main;

namespace UltimaCR.Rotations
{
    public sealed partial class Astrologian
    {
        private AstrologianSpells _mySpells;

        private AstrologianSpells MySpells
        {
            get { return _mySpells ?? (_mySpells = new AstrologianSpells()); }
        }

        #region Job Spells

        //Posture
        private async Task<bool> NocturnalSect()
        {
            if (Ultima.UltSettings.NocturnalSect &&
                !Core.Player.HasAura(MySpells.NocturnalSect.Name))
            {
                return await MySpells.NocturnalSect.Cast();
            }
            return false;
        }
        private async Task<bool> DiurnalSect()
        {
            if (Ultima.UltSettings.DiurnalSect &&
                !Core.Player.HasAura(MySpells.DiurnalSect.Name))
            {
                return await MySpells.DiurnalSect.Cast();
            }
            return false;
        }
        //Heal Region
        private async Task<bool> AspectedBenefic()
        {
            if (Ultima.UltSettings.AstrologianAspectedBenefic)
            {
                if (!Core.Player.HasAura(MySpells.DiurnalSect.Name))
                {
                    var target = Helpers.HealManager.FirstOrDefault(hm =>
                    !hm.HasAura(837) &&
                    (hm.IsHealer() && hm.CurrentHealthPercent <= 70 ||
                    hm.IsTank() && hm.CurrentHealthPercent <= 70));

                    if (target != null)
                    {
                        return await MySpells.AspectedBenefic.Cast(target);
                    }
                }


                if (!Core.Player.HasAura(MySpells.NocturnalSect.Name))
                {
                    var target = Helpers.HealManager.FirstOrDefault(hm =>
                    !hm.HasAura(835) &&
                    (hm.IsHealer() && hm.CurrentHealthPercent <= 80 ||
                    hm.IsTank() && hm.CurrentHealthPercent <= 90));

                    if (target != null)
                    {
                        return await MySpells.AspectedBenefic.Cast(target);
                    }
                }
            }
            return false;
        }
        private async Task<bool> Benefic()
        {
            if (Ultima.UltSettings.AstrologianBenefic)
            {
                if (!Core.Player.HasAura(815))
                {
                    var target = Helpers.HealManager.FirstOrDefault(hm =>
                        !Core.Player.HasAura(MySpells.DiurnalSect.Name) &&
                        hm.CurrentHealthPercent <= 80);

                    if (target != null)
                    {
                        return await MySpells.Benefic.Cast(target);
                    }
                }
                else
                {
                    var target = Helpers.HealManager.FirstOrDefault(hm =>
                         !Core.Player.HasAura(MySpells.DiurnalSect.Name) &&
                         hm.CurrentHealthPercent <= 90);

                    if (target != null)
                    {
                        return await MySpells.BeneficII.Cast(target);
                    }
                }
            }

            {
                if (!Core.Player.HasAura(815))
                {
                    var target = Helpers.HealManager.FirstOrDefault(hm =>
                        !Core.Player.HasAura(MySpells.NocturnalSect.Name) &&
                         hm.CurrentHealthPercent <= 80);

                    if (target != null)
                    {
                        return await MySpells.Benefic.Cast(target);
                    }
                }
                else
                {
                    var target = Helpers.HealManager.FirstOrDefault(hm =>
                        !Core.Player.HasAura(MySpells.NocturnalSect.Name) &&
                         hm.CurrentHealthPercent <= 90);

                    if (target != null)
                    {
                        return await MySpells.BeneficII.Cast(target);
                    }
                }
            }
            return false;
        }
        private async Task<bool> BeneficII()
        {
            if (Ultima.UltSettings.AstrologianBeneficII)
            {
                var target = Helpers.HealManager.FirstOrDefault(hm =>
                !Core.Player.HasAura(MySpells.DiurnalSect.Name) &&
                hm.IsHealer() && hm.CurrentHealthPercent <= 60 ||
                hm.IsTank() && hm.CurrentHealthPercent <= 60);

                if (target != null)
                {
                    return await MySpells.BeneficII.Cast(target);
                }
            }

            {
                var target = Helpers.HealManager.FirstOrDefault(hm =>
                !Core.Player.HasAura(MySpells.NocturnalSect.Name) &&
                hm.IsHealer() && hm.CurrentHealthPercent <= 60 ||
                hm.IsTank() && hm.CurrentHealthPercent <= 60);

                if (target != null)
                {
                    return await MySpells.BeneficII.Cast(target);
                }
            }
            return false;
        }
        private async Task<bool> Helios()
        {
            if (!Core.Player.HasAura(MySpells.DiurnalSect.Name))
            {
                if (Helpers.HealManager.Count(hm =>
                hm.Distance2D(Core.Player) - hm.CombatReach - Core.Player.CombatReach <= 15 &&
                (hm.CurrentHealthPercent <= 70)) >= 3)
                {
                    return await MySpells.Helios.Cast();
                }
            }
            if (!Core.Player.HasAura(MySpells.NocturnalSect.Name))
            {
                if (Helpers.HealManager.Count(hm =>
                hm.Distance2D(Core.Player) - hm.CombatReach - Core.Player.CombatReach <= 15 &&
                (hm.CurrentHealthPercent <= 80)) >= 3)
                {
                    return await MySpells.Helios.Cast();
                }
            }
            return false;
        }
        private async Task<bool> AspectedHelios()
        {
            if (!Core.Player.HasAura(MySpells.DiurnalSect.Name))
            {
                if (Helpers.HealManager.Count(hm =>
                !hm.HasAura(837) &&
                hm.Distance2D(Core.Player) - hm.CombatReach - Core.Player.CombatReach <= 15 &&
                (hm.CurrentHealthPercent <= 70)) >= 3)
                {
                    return await MySpells.AspectedHelios.Cast();
                }
            }

            if (!Core.Player.HasAura(MySpells.NocturnalSect.Name))
            {
                if (Helpers.HealManager.Count(hm =>
                !hm.HasAura(MySpells.AspectedHelios.Name) &&
                hm.Distance2D(Core.Player) - hm.CombatReach - Core.Player.CombatReach <= 15 &&
                (hm.CurrentHealthPercent <= 80)) >= 3)
                {
                    return await MySpells.AspectedHelios.Cast();
                }
            }
            return false;
        }
        private async Task<bool> EssentialDignity()
        {
            if (Ultima.UltSettings.AstrologianEssentialDignity)
            {
                var target = Helpers.HealManager.FirstOrDefault(hm =>
                    hm.IsHealer() && hm.CurrentHealthPercent <= 30 ||
                    hm.IsTank() && hm.CurrentHealthPercent <= 30);

                if (target != null)
                {
                    return await MySpells.EssentialDignity.Cast(target);
                }
            }
            return false;
        }
        private async Task<bool> Synastry()
        {
            var target = Helpers.HealManager.FirstOrDefault(hm =>
                hm.IsHealer() && hm.CurrentHealthPercent <= 80 &&
                hm.IsTank() && hm.CurrentHealthPercent <= 80);

            if (target != null)
            {
                return await MySpells.Synastry.Cast(target);
            }
            return false;
        }
        private async Task<bool> CollectiveUnconscious()
        {
            {
                var target = Helpers.HealManager.FirstOrDefault(pm1 =>
                Helpers.HealManager.Count(pm2 =>
                pm1.Distance2D(pm2) - pm1.CombatReach - pm2.CombatReach <= 4 &&
                pm2.CurrentHealthPercent <= 80) >= 3);

                if (target != null)
                {
                    return await MySpells.CollectiveUnconscious.Cast(target);
                }
            }
            return false;
        }
        private async Task<bool> Ascend()
        {
            if (!Helpers.HealManager.Any(hm => hm.CurrentHealthPercent <= 70))
            {
                var target = Helpers.PartyMembers.FirstOrDefault(pm =>
                    pm.IsDead &&
                    pm.Type == GameObjectType.Pc &&
                    !pm.HasAura(MySpells.CrossClass.Raise.Name));

                if (target != null &&
                    Actionmanager.CanCast(MySpells.Ascend.Name, target))
                {
                    if (Actionmanager.CanCast(MySpells.CrossClass.Swiftcast.Name, Core.Player))
                    {
                        if (await MySpells.CrossClass.Swiftcast.Cast())
                        {
                            await Coroutine.Wait(3000, () => Actionmanager.CanCast(MySpells.Ascend.Name, target) &&
                                                             Core.Player.HasAura(MySpells.CrossClass.Swiftcast.Name));
                        }
                    }
                    return await MySpells.Ascend.Cast(target);
                }
            }
            return false;
        }
        //Combat Buff
        private async Task<bool> Lightspeed()
        {
            if (Helpers.HealManager.Count(hm => hm.CurrentHealthPercent <= 70) >= 3)
            {
                return await MySpells.Lightspeed.Cast();
            }
            return false;
        }
        private async Task<bool> LuminiferousAether()
        {
            if (Core.Player.CurrentManaPercent <= 70)
            {
                return await MySpells.LuminiferousAether.Cast();
            }
            return false;
        }
        private async Task<bool> TimeDilation()
        {
            return await MySpells.TimeDilation.Cast();
        }
        //Combat
        private async Task<bool> Malefic()
        {
            return await MySpells.Malefic.Cast();
        }
        private async Task<bool> MaleficII()
        {
            return await MySpells.MaleficII.Cast();
        }
        private async Task<bool> Combust()
        {
            if (!Core.Player.CurrentTarget.HasAura(MySpells.Combust.Name, true, 4000))
            {
                return await MySpells.Combust.Cast();
            }
            return false;
        }
        private async Task<bool> CombustII()
        {
            if (!Core.Player.CurrentTarget.HasAura(MySpells.CombustII.Name, true, 4000))
            {
                return await MySpells.CombustII.Cast();
            }
            return false;
        }
        private async Task<bool> Gravity()
        {
            return await MySpells.Gravity.Cast();
        }
        private async Task<bool> CelestialOpposition()
        {
            return await MySpells.CelestialOpposition.Cast();
        }
        //Dot
        private async Task<bool> ExaltedDetriment()
        {
            return await MySpells.ExaltedDetriment.Cast();
        }
        private async Task<bool> Stella()
        {
            return await MySpells.Stella.Cast();
        }
        private async Task<bool> Disable()
        {
            if (!Core.Player.CurrentTarget.HasAura(MySpells.Disable.Name, true))
            {
                var target = Helpers.HealManager.FirstOrDefault(hm =>
                    Helpers.EnemiesNearTarget(8) <= 1 &&
                    hm.IsTank() && hm.CurrentHealthPercent <= 90);

                if (target != null)
                {
                    return await MySpells.Disable.Cast();
                }
            }
            return false;
        }
        //Card Draw
        private async Task<bool> Draw()
        {
            if (Ultima.UltSettings.AstrologianDraw)
            {
                return await MySpells.Draw.Cast();
            }
            return false;
        }
        private async Task<bool> RoyalRoad()
        {
            return await MySpells.RoyalRoad.Cast();
        }
        private async Task<bool> Spread()
        {
            return await MySpells.Spread.Cast();
        }

        private async Task<bool> Shuffle()
        {
            return await MySpells.Shuffle.Cast();
        }

        #endregion

        #region Cross Class Spells

        #region Conjurer

        private async Task<bool> Cure()
        {
            if (Ultima.UltSettings.AstrologianCure)
            {
                return await MySpells.CrossClass.Cure.Cast();
            }
            return false;
        }
        private async Task<bool> Aero()
        {
            if (Ultima.UltSettings.AstrologianAero &&
                !Core.Player.CurrentTarget.HasAura(MySpells.CrossClass.Aero.Name, true, 4000))
            {
                return await MySpells.CrossClass.Aero.Cast();
            }
            return false;
        }
        private async Task<bool> ClericStance()
        {
            if (Ultima.UltSettings.AstrologianClericStance)
            {
                return await MySpells.CrossClass.ClericStance.Cast();
            }
            return false;
        }
        private async Task<bool> Protect()
        {
            if (Ultima.UltSettings.AstrologianProtect &&
                !Helpers.HealManager.Any(hm => hm.CurrentHealthPercent <= 70 || hm.IsDead))
            {
                var target = Helpers.HealManager.FirstOrDefault(hm => hm.Type == GameObjectType.Pc && !hm.HasAura(MySpells.CrossClass.Protect.Name));

                if (target != null)
                {
                    return await MySpells.CrossClass.Protect.Cast(target);
                }
            }
            return false;
        }
        private async Task<bool> Raise()
        {
            if (Ultima.UltSettings.AstrologianRaise)
            {
                return await MySpells.CrossClass.Raise.Cast();
            }
            return false;
        }
        private async Task<bool> Stoneskin()
        {
            if (Ultima.UltSettings.AstrologianStoneskin &&
                !Core.Player.HasAura(MySpells.CrossClass.Stoneskin.Name))
            {
                var target = Helpers.HealManager.FirstOrDefault(hm => hm.Type == GameObjectType.Pc && !hm.HasAura(MySpells.CrossClass.Stoneskin.Name));

                if (target != null)
                {
                    return await MySpells.CrossClass.Stoneskin.Cast(target);
                }
            }
            return false;
        }

        #endregion

        #region Thaumaturge

        private async Task<bool> Surecast()
        {
            if (Ultima.UltSettings.AstrologianSurecast)
            {
                return await MySpells.CrossClass.Surecast.Cast();
            }
            return false;
        }

        private async Task<bool> BlizzardII()
        {
            if (Ultima.UltSettings.AstrologianBlizzardII)
            {
                return await MySpells.CrossClass.BlizzardII.Cast();
            }
            return false;
        }

        private async Task<bool> Swiftcast()
        {
            if (Ultima.UltSettings.AstrologianSwiftcast)
            {
                return await MySpells.CrossClass.Swiftcast.Cast();
            }
            return false;
        }

        #endregion

        #endregion

        #region PvP Spells



        #endregion

        #region Custom Spells
        private async Task<bool> Play()
        {
            if (Ultima.UltSettings.AstrologianDraw)
            {
                if (Core.Player.HasAura(913))
                {
                    return await MySpells.TheBalance.Cast();
                }
                if (Core.Player.HasAura(914))
                {
                    return await MySpells.TheBole.Cast();
                }
                if (Core.Player.HasAura(915))
                {
                    return await MySpells.TheArrow.Cast();
                }
                if (Core.Player.HasAura(916))
                {
                    return await MySpells.TheSpear.Cast();
                }
                if (Core.Player.HasAura(917))
                {
                    return await MySpells.TheEwer.Cast();
                }
                if (Core.Player.HasAura(918))
                {
                    return await MySpells.TheSpire.Cast();
                }
            }
            return false;
        }
        #endregion
    }
}