﻿using System;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Managers;
using System.Linq;
using System.Threading.Tasks;
using UltimaCR.Spells.Main;

namespace UltimaCR.Rotations
{
    public sealed partial class Summoner
    {
        private SummonerSpells _mySpells;

        private SummonerSpells MySpells
        {
            get { return _mySpells ?? (_mySpells = new SummonerSpells()); }
        }

        #region Class Spells
        private async Task<bool> Summon()
        {
            if (Core.Player.Pet == null &&
                Ultima.UltSettings.SummonerSummonPet)
            {
                if (Ultima.UltSettings.SummonerGaruda ||
                    !Actionmanager.HasSpell(MySpells.SummonII.Name) &&
                    Ultima.UltSettings.SummonerTitan ||
                    !Actionmanager.HasSpell(MySpells.SummonIII.Name) &&
                    Ultima.UltSettings.SummonerIfrit)
                {
                    if (Actionmanager.CanCast(MySpells.CrossClass.Swiftcast.Name, Core.Player))
                    {
                        if (await MySpells.CrossClass.Swiftcast.Cast())
                        {
                            await Coroutine.Wait(3000, () => Actionmanager.CanCast(MySpells.Summon.Name, Core.Player) &&
                                                             Core.Player.HasAura(MySpells.CrossClass.Swiftcast.Name));
                        }
                    }
                    if (await MySpells.Summon.Cast())
                    {
                        await Coroutine.Wait(2000, () => Actionmanager.CanCast(MySpells.Summon.Name, Core.Player));
                    }
                }
                return true;
            }
            return false;
        }

        private async Task<bool> SummonII()
        {
            if (Core.Player.Pet == null &&
                Ultima.UltSettings.SummonerTitan)
            {
                if (Ultima.UltSettings.SummonerTitan ||
                    !Actionmanager.HasSpell(MySpells.Summon.Name) &&
                    Ultima.UltSettings.SummonerGaruda ||
                    !Actionmanager.HasSpell(MySpells.SummonIII.Name) &&
                    Ultima.UltSettings.SummonerIfrit)
                {
                    if (Actionmanager.CanCast(MySpells.CrossClass.Swiftcast.Name, Core.Player))
                    {
                        if (await MySpells.CrossClass.Swiftcast.Cast())
                        {
                            await Coroutine.Wait(3000, () => Actionmanager.CanCast(MySpells.SummonII.Name, Core.Player) &&
                                                             Core.Player.HasAura(MySpells.CrossClass.Swiftcast.Name));
                        }
                    }
                    if (await MySpells.SummonII.Cast())
                    {
                        await Coroutine.Wait(2000, () => Actionmanager.CanCast(MySpells.SummonII.Name, Core.Player));
                    }
                }
                return true;
            }
            return false;
        }

        private async Task<bool> Ruin()
        {
            if (!Core.Player.HasAura("Dreadwyrm Trance", true))
            {
                if (Actionmanager.CanCast(MySpells.Aetherflow.Name, Core.Player) &&
                    !Core.Player.HasAura(MySpells.Aetherflow.Name) ||
                    Core.Player.CurrentTarget.HasAura(MySpells.BioII.Name, true) &&
                    Core.Player.CurrentTarget.HasAura(MySpells.Miasma.Name, true) &&
                    Core.Player.CurrentTarget.HasAura(MySpells.Bio.Name, true))
                {
                    if (await MySpells.Ruin.Cast())
                    {
                        await Coroutine.Wait(3000, () => Actionmanager.CanCast(MySpells.Ruin.Name, Core.Player));
                    }
                    return false;
                }
            }
            return false;
        }

        private async Task<bool> RuinII()
        {
            if (MovementManager.IsMoving)
            {
                return await MySpells.RuinII.Cast();
            }
            if (Core.Player.CurrentManaPercent > 40)
            {
                if (Actionmanager.CanCast(MySpells.Aetherflow.Name, Core.Player) &&
                    !Core.Player.HasAura(MySpells.Aetherflow.Name) ||
                    Actionmanager.CanCast(MySpells.Fester.Name, Core.Player.CurrentTarget) &&
                    Helpers.EnemiesNearTarget(8) <= 1 &&
                    Ultima.UltSettings.SmartTarget ||
                    Actionmanager.CanCast(MySpells.Fester.Name, Core.Player.CurrentTarget) &&
                    Ultima.UltSettings.SingleTarget ||
                    Actionmanager.CanCast(MySpells.Bane.Name, Core.Player.CurrentTarget) &&
                    !Ultima.UltSettings.SingleTarget &&
                    Helpers.EnemiesNearTarget(8) > 1 &&
                    Core.Player.CurrentTarget.HasAura(MySpells.BioII.Name, true, 8000) &&
                    Core.Player.CurrentTarget.HasAura(MySpells.Miasma.Name, true, 8000) &&
                    Core.Player.CurrentTarget.HasAura(MySpells.Bio.Name, true, 5000) ||
                    Actionmanager.CanCast(MySpells.Rouse.Name, Core.Player) ||
                    Actionmanager.CanCast(MySpells.Spur.Name, Core.Player))
                {
                    return await MySpells.RuinII.Cast();
                }
            }
            return false;
        }

        //Defensive
        private async Task<bool> EyeForAnEye()
        {
            var target = Helpers.HealManager.FirstOrDefault(hm =>
            !hm.HasAura(185) &&
            hm.IsTank() && hm.CurrentHealthPercent <= 90);

            if (target != null)
            {
                return await MySpells.EyeForAnEye.Cast(target);
            }
            return false;
        }

        //Heal
        private async Task<bool> Physick()
        {
            if (Ultima.UltSettings.SummonerPhysick)
            {
                if (Core.Player.CurrentHealthPercent < 20)
                {
                    return await MySpells.Physick.Cast();
                }
            }
            return false;
        }

        private async Task<bool> Sustain()
        {
            if (Core.Player.Pet != null &&
                Core.Player.Pet.CurrentHealthPercent < 70 &&
                !Core.Player.Pet.HasAura(MySpells.Sustain.Name))
            {
                return await MySpells.Sustain.Cast();
            }
            return false;
        }

        private async Task<bool> Resurrection()
        {
            if (!Helpers.HealManager.Any(hm => hm.CurrentHealthPercent <= 70))
            {
                var target = Helpers.PartyMembers.FirstOrDefault(pm =>
                    pm.IsDead &&
                    pm.Type == GameObjectType.Pc &&
                    !pm.HasAura("Raise"));

                if (target != null &&
                    Actionmanager.CanCast(MySpells.Resurrection.Name, target))
                {
                    if (Actionmanager.CanCast(MySpells.CrossClass.Swiftcast.Name, Core.Player))
                    {
                        if (await MySpells.CrossClass.Swiftcast.Cast())
                        {
                            await Coroutine.Wait(3000, () => Actionmanager.CanCast(MySpells.Resurrection.Name, target) &&
                                                             Core.Player.HasAura(MySpells.CrossClass.Swiftcast.Name));
                        }
                    }
                    return await MySpells.Resurrection.Cast(target);
                }
            }
            return false;
        }

        //DoTs
        private async Task<bool> Bio()
        {
            if (!Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true))
            {
                if (!Core.Player.CurrentTarget.HasAura(MySpells.Bio.Name, true, 3000))
                {
                    return await MySpells.Bio.Cast();
                }
            }
            return false;
        }

        private async Task<bool> BioII()
        {
            if (!Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true))
            {
                if (!Core.Player.CurrentTarget.HasAura(MySpells.BioII.Name, true, 5000))
                {
                    return await MySpells.BioII.Cast();
                }
            }
            return false;
        }

        private async Task<bool> Miasma()
        {
            if (!Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true))
            {
                if (!Core.Player.CurrentTarget.HasAura(MySpells.Miasma.Name, true, 5000))
                {
                    return await MySpells.Miasma.Cast();
                }
            }
            return false;
        }

        //AOE
        private async Task<bool> ShadowFlare()
        {
            if (!Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true) &&
                Ultima.UltSettings.SingleTarget ||
                !Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true) &&
                Helpers.EnemiesNearTarget(5) >= 3 &&
                Ultima.UltSettings.SmartTarget ||
                !Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true) &&
                Ultima.UltSettings.MultiTarget)
            {
                if (Actionmanager.HasSpell(MySpells.ShadowFlare.Name) &&
                !Core.Player.HasAura(MySpells.ShadowFlare.Name, true, 4000))
                {
                    if (Actionmanager.CanCast(MySpells.RuinII.Name, Core.Player.CurrentTarget) &&
                        Actionmanager.CanCast(MySpells.CrossClass.Swiftcast.Name, Core.Player))
                    {
                        if (await MySpells.RuinII.Cast())
                        {
                            await Coroutine.Wait(3000, () => Actionmanager.CanCast(MySpells.CrossClass.Swiftcast.Name, Core.Player));
                        }
                    }
                    if (Actionmanager.CanCast(MySpells.CrossClass.Swiftcast.Name, Core.Player))
                    {
                        if (await MySpells.CrossClass.Swiftcast.Cast())
                        {
                            await Coroutine.Wait(3000, () => Actionmanager.CanCast(MySpells.ShadowFlare.Name, Core.Player.CurrentTarget) &&
                                                             Core.Player.HasAura(MySpells.CrossClass.Swiftcast.Name));
                        }
                    }
                    if (await MySpells.ShadowFlare.Cast())
                    {
                        await Coroutine.Wait(3000, () => Core.Player.HasAura(MySpells.ShadowFlare.Name));
                    }
                    return true;
                }
            }
            return false;
        }

        private async Task<bool> MiasmaII()
        {
            if (!Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true))
            {
                if (Helpers.EnemiesNearTarget(5) >= 3 &&
                    Core.Player.TargetDistance(5, false) &&
                    Ultima.UltSettings.SmartTarget ||
                    Ultima.UltSettings.MultiTarget &&
                    Core.Player.TargetDistance(5, false) ||
                    Ultima.UltSettings.SingleTarget &&
                    Core.Player.TargetDistance(5, false) &&
                    !Core.Player.CurrentTarget.HasAura(MySpells.MiasmaII.Name, true, 2000))
                {
                    return await MySpells.MiasmaII.Cast();
                }
            }
            return false;
        }

        private async Task<bool> Bane()
        {
            if (Helpers.EnemiesNearTarget(8) >= 3 &&
                Ultima.UltSettings.SmartTarget ||
                Ultima.UltSettings.MultiTarget &&
                Core.Player.HasAura(MySpells.Aetherflow.Name) &&
                Core.Player.CurrentTarget.HasAura(MySpells.BioII.Name, true, 8000) &&
                Core.Player.CurrentTarget.HasAura(MySpells.Miasma.Name, true, 8000) &&
                Core.Player.CurrentTarget.HasAura(MySpells.Bio.Name, true, 5000))
            {
                return await MySpells.Bane.Cast();
            }
            return false;
        }

        //Buff
        private async Task<bool> Aetherflow()
        {
            if (!Core.Player.HasAura(MySpells.Aetherflow.Name))
            {
                return await MySpells.Aetherflow.Cast();
            }
            return false;
        }

        //Buff pet
        private async Task<bool> Rouse()
        {
            return await MySpells.Rouse.Cast();
        }

        //Debuff
        private async Task<bool> Virus()
        {
            if (!Core.Player.CurrentTarget.HasAura(410, true))
            {
                var target = Helpers.HealManager.FirstOrDefault(hm =>
                    Helpers.EnemiesNearPlayer(25) >= 1 &&
                    hm.IsTank() && hm.CurrentHealthPercent <= 80);

                if (target != null)
                {
                    return await MySpells.Virus.Cast();
                }
            }
            return false;
        }

        //Cooldown
        private async Task<bool> EnergyDrain()
        {
            if (Core.Player.CurrentManaPercent <= 70)
            {
                return await MySpells.EnergyDrain.Cast();
            }
            return false;
        }

        #endregion

        #region Cross Class Spells

        #region Archer

        //Buff
        private async Task<bool> RagingStrikes()
        {
            if (Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true))
            {
                return await MySpells.CrossClass.RagingStrikes.Cast();
            }
            return false;
        }

        private async Task<bool> HawksEye()
        {
            if (Ultima.LastSpell.Name != MySpells.Painflare.Name)
            {
                return await MySpells.CrossClass.HawksEye.Cast();
            }
            return false;
        }

        //Defensive
        private async Task<bool> QuellingStrikes()
        {
            return await MySpells.CrossClass.QuellingStrikes.Cast();
        }

        #endregion

        #region Thaumaturge

        private async Task<bool> Surecast()
        {
            if (Ultima.UltSettings.SummonerSurecast)
            {
                return await MySpells.CrossClass.Surecast.Cast();
            }
            return false;
        }

        private async Task<bool> BlizzardII()
        {
            if (Ultima.UltSettings.SummonerBlizzardII)
            {
                return await MySpells.CrossClass.BlizzardII.Cast();
            }
            return false;
        }

        private async Task<bool> Swiftcast()
        {
            if (Ultima.UltSettings.SummonerSwiftcast)
            {
                return await MySpells.CrossClass.Swiftcast.Cast();
            }
            return false;
        }

        #endregion

        #endregion

        #region Job Spells

        private async Task<bool> RuinIII()
        {
            if (Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true))
            {
                if (await MySpells.RuinIII.Cast())
                {
                    await Coroutine.Wait(2400, () => Actionmanager.CanCast(MySpells.RuinIII.Name, Core.Player.CurrentTarget));
                }
                return false;
            }
            return false;
        }

        private async Task<bool> SummonIII()
        {
            if (Core.Player.Pet == null &&
                Ultima.UltSettings.SummonerIfrit)
            {
                if (Ultima.UltSettings.SummonerIfrit ||
                    !Actionmanager.HasSpell(MySpells.SummonII.Name) &&
                    Ultima.UltSettings.SummonerTitan ||
                    !Actionmanager.HasSpell(MySpells.Summon.Name) &&
                    Ultima.UltSettings.SummonerGaruda)
                {
                    if (Actionmanager.CanCast(MySpells.CrossClass.Swiftcast.Name, Core.Player))
                    {
                        if (await MySpells.CrossClass.Swiftcast.Cast())
                        {
                            await Coroutine.Wait(3000, () => Actionmanager.CanCast(MySpells.SummonIII.Name, Core.Player) &&
                                                             Core.Player.HasAura(MySpells.CrossClass.Swiftcast.Name));
                        }
                    }
                    if (await MySpells.SummonIII.Cast())
                    {
                        await Coroutine.Wait(2000, () => Actionmanager.CanCast(MySpells.SummonIII.Name, Core.Player));
                    }
                }
                return true;
            }
            return false;
        }

        //AOE Pet >> Titan >> Garuda >> Ifrit
        private async Task<bool> MountainBuster()
        {
            if (Ultima.UltSettings.SingleTarget ||
                Helpers.EnemiesNearTarget(5) >= 3 &&
                Ultima.UltSettings.SmartTarget ||
                Ultima.UltSettings.MultiTarget &&
                Core.Player.Pet != null &&
                Core.Player.Pet.Name == "Titan-Egi")
            {
                return await MySpells.MountainBuster.Cast();
            }
            return false;
        }

        private async Task<bool> AerialSlash()
        {
            if (Ultima.UltSettings.SingleTarget ||
                Helpers.EnemiesNearTarget(5) >= 3 &&
                Ultima.UltSettings.SmartTarget ||
                Ultima.UltSettings.MultiTarget &&
                Core.Player.Pet != null &&
                Core.Player.Pet.Name == "Garuda-Egi")
            {
                return await MySpells.AerialSlash.Cast();
            }
            return false;
        }

        private async Task<bool> FlamingCrush()
        {
            if (Ultima.UltSettings.SingleTarget ||
                Helpers.EnemiesNearTarget(5) >= 3 &&
                Ultima.UltSettings.SmartTarget ||
                Ultima.UltSettings.MultiTarget &&
                Core.Player.Pet != null &&
                Core.Player.Pet.Name == "Ifrit-Egi")
            {
                return await MySpells.FlamingCrush.Cast();
            }
            return false;
        }

        //Defensive Pet >> Titan >> Ifrit
        private async Task<bool> EarthenWard()
        {
            if (Core.Player.Pet != null &&
                Core.Player.Pet.Name == "Titan-Egi" &&
                Core.Player.Pet.CurrentHealthPercent <= 80)
            {
                return await MySpells.EarthenWard.Cast();
            }
            return false;
        }

        private async Task<bool> RadiantShield()
        {
            if (Core.Player.Pet != null &&
                Core.Player.Pet.Name == "Ifrit-Egi" &&
                Core.Player.Pet.CurrentHealthPercent <= 80)
            {
                return await MySpells.RadiantShield.Cast();
            }
            return false;
        }

        //Buff Garuda
        private async Task<bool> Contagion()
        {
            if (!Actionmanager.CanCast(MySpells.Tridisaster.Name, Core.Player.CurrentTarget) &&
                Core.Player.CurrentTarget.HasAura(MySpells.BioII.Name, true, 2000) &&
                Core.Player.CurrentTarget.HasAura(MySpells.Miasma.Name, true, 2000) &&
                Core.Player.CurrentTarget.HasAura(MySpells.Bio.Name, true, 2000))
            {
                if (Core.Player.Pet != null &&
                    Core.Player.Pet.Name == "Garuda-Egi")
                {
                    return await MySpells.Contagion.Cast();
                }
            }
            return false;
        }

        //Attaque pet
        private async Task<bool> Landslide()
        {
            if (Core.Player.Pet != null &&
                Core.Player.Pet.Name == "Titan-Egi")
            {
                return await MySpells.Landslide.Cast();
            }
            return false;
        }

        private async Task<bool> CrimsonCyclone()
        {
            if (Core.Player.Pet != null &&
                Core.Player.Pet.Name == "Ifrit-Egi")
            {
                return await MySpells.CrimsonCyclone.Cast();
            }
            return false;
        }

        private async Task<bool> Shockwave()
        {
            if (Core.Player.Pet != null &&
                Core.Player.Pet.Name == "Garuda-Egi" &&
                Core.Player.TargetDistance(5, false))
            {
                return await MySpells.Shockwave.Cast();
            }
            return false;
        }

        //Buff Pet
        private async Task<bool> Spur()
        {
            return await MySpells.Spur.Cast();
        }

        //Cooldown Pet
        private async Task<bool> Enkindle()
        {
            if (!Actionmanager.CanCast(MySpells.Rouse.Name, Core.Player) &&
                !Actionmanager.CanCast(MySpells.Spur.Name, Core.Player))
            {
                return await MySpells.Enkindle.Cast();
            }
            return false;
        }

        //Buff
        private async Task<bool> DreadwyrmTrance()
        {
            if (await MySpells.DreadwyrmTrance.Cast())
            {
                await Coroutine.Wait(2400, () => Actionmanager.CanCast(MySpells.DreadwyrmTrance.Name, Core.Player));
            }
            return false;
        }

        //Cooldown

        private async Task<bool> Fester()
        {
            if (Core.Player.HasAura(MySpells.Aetherflow.Name) &&
                Helpers.EnemiesNearTarget(8) < 3 &&
                Ultima.UltSettings.SmartTarget ||
                Ultima.UltSettings.SingleTarget &&
                Core.Player.CurrentTarget.HasAura(MySpells.BioII.Name) &&
                Core.Player.CurrentTarget.HasAura(MySpells.Miasma.Name) &&
                Core.Player.CurrentTarget.HasAura(MySpells.Bio.Name))
                {
                    if (await MySpells.Fester.Cast())
                    {
                        await Coroutine.Wait(1000, () => Actionmanager.CanCast(MySpells.Fester.Name, Core.Player));
                    }
                    return false;
                }
            return false;
        }

        private async Task<bool> Deathflare()
        {
            if (!Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true, 5000) ||
                Core.Player.CurrentTarget.CurrentHealthPercent <= 20 &&
                Ultima.UltSettings.SmartTarget ||
                Core.Player.CurrentTarget.CurrentHealthPercent <= 10 &&
                Ultima.UltSettings.SingleTarget ||
                Core.Player.CurrentTarget.CurrentHealthPercent <= 25 &&
                Ultima.UltSettings.MultiTarget)
            {
                if (await MySpells.Deathflare.Cast())
                {
                    await Coroutine.Wait(2400, () => Actionmanager.CanCast(MySpells.Deathflare.Name, Core.Player.CurrentTarget));
                }
                return false;
            }
            return false;
        }

        //AOE
        private async Task<bool> Painflare()
        {
            if (Core.Player.HasAura(MySpells.Aetherflow.Name) &&
                Ultima.UltSettings.SingleTarget &&
                Ultima.LastSpell.Name == MySpells.Fester.Name ||
                Helpers.EnemiesNearTarget(5) >= 3 &&
                Ultima.UltSettings.SmartTarget ||
                Ultima.UltSettings.MultiTarget)
            {
                return await MySpells.Painflare.Cast();
            }
            return false;
        }

        //Debuff
        private async Task<bool> Tridisaster()
        {
            if (!Core.Player.HasAura(MySpells.DreadwyrmTrance.Name, true))
            {
                if (!Core.Player.CurrentTarget.HasAura(MySpells.BioII.Name, true, 5000) ||
                    !Core.Player.CurrentTarget.HasAura(MySpells.Miasma.Name, true, 3000))
                {
                    if (await MySpells.Tridisaster.Cast())
                    {
                        await Coroutine.Wait(2400, () => Actionmanager.CanCast(MySpells.Tridisaster.Name, Core.Player.CurrentTarget));
                    }
                }
            }
            return false;
        }

        private async Task<bool> Tribind()
        {
            return await MySpells.Tribind.Cast();
        }

        #endregion

        #region PvP Spells

        private async Task<bool> AethericBurst()
        {
            return await MySpells.PvP.AethericBurst.Cast();
        }

        private async Task<bool> Equanimity()
        {
            return await MySpells.PvP.Equanimity.Cast();
        }

        private async Task<bool> ManaDraw()
        {
            return await MySpells.PvP.ManaDraw.Cast();
        }

        private async Task<bool> MistyVeil()
        {
            return await MySpells.PvP.MistyVeil.Cast();
        }

        private async Task<bool> Purify()
        {
            return await MySpells.PvP.Purify.Cast();
        }

        private async Task<bool> Wither()
        {
            return await MySpells.PvP.Wither.Cast();
        }

        #endregion
    }
}